@@ -22,32 +22,47 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 #  USA
 
+from __future__ import absolute_import, print_function
+
 import apt_pkg
+import copy
 from hashlib import md5
 import re
 import os
 import glob
-import sys
 import shutil
-import subprocess
 import threading
 import atexit
 import tempfile
-import string
+try:
+  from string import maketrans
+except ImportError:
+  maketrans = str.maketrans
 import stat
 
 from tempfile import NamedTemporaryFile
 from xml.sax.saxutils import escape
-from ConfigParser import ConfigParser
+try:
+  from configparser import ConfigParser
+except ImportError:
+  from ConfigParser import ConfigParser
 from gettext import gettext as _
-from urlparse import urlparse
 
-from AptAuth import AptAuth
-import softwareproperties
 import aptsources
 import aptsources.distro
+import softwareproperties
+
+from .AptAuth import AptAuth
 from aptsources.sourceslist import SourcesList, SourceEntry
-from ppa import AddPPASigningKeyThread, expand_ppa_line
+from . import shortcuts
+from . import ppa
+from . import cloudarchive
+
+_SHORTCUT_FACTORIES = [
+    ppa.shortcut_handler,
+    cloudarchive.shortcut_handler,
+    shortcuts.shortcut_handler,
+]
 
 
 class SoftwareProperties(object):
@@ -59,21 +74,25 @@ class SoftwareProperties(object):
   RELEASE_UPGRADES_CONF = "/etc/update-manager/release-upgrades"
   #RELEASE_UPGRADES_CONF = "/tmp/release-upgrades"
   (
-    RELEASE_UPGRADES_NEVER,
     RELEASE_UPGRADES_NORMAL,
-    RELEASE_UPGRADES_LTS
-  ) = range(3)
+    RELEASE_UPGRADES_LTS,
+    RELEASE_UPGRADES_NEVER
+  ) = list(range(3))
   release_upgrades_policy_map = {
-    RELEASE_UPGRADES_NEVER  : 'never',
     RELEASE_UPGRADES_NORMAL : 'normal',
     RELEASE_UPGRADES_LTS    : 'lts',
+    RELEASE_UPGRADES_NEVER  : 'never',
   }
   
-  def __init__(self, datadir=None, options=None):
+  def __init__(self, datadir=None, options=None, rootdir="/"):
     """ Provides the core functionality to configure the used software 
         repositories, the corresponding authentication keys and 
         update automation """
-    self.popconfile = "/etc/popularity-contest.conf"
+    self.popconfile = rootdir+"/etc/popularity-contest.conf"
+
+    self.rootdir = rootdir
+    if rootdir != "/":
+      apt_pkg.config.set("Dir", rootdir)
 
     # FIXME: some saner way is needed here
     if datadir == None:
@@ -97,7 +116,7 @@ class SoftwareProperties(object):
     self.custom_mirrors= []
 
     # apt-key stuff
-    self.apt_key = AptAuth()
+    self.apt_key = AptAuth(rootdir=rootdir)
 
     atexit.register(self.wait_for_threads)
 
@@ -128,7 +147,7 @@ class SoftwareProperties(object):
     if apt_pkg.config.find_i(softwareproperties.CONF_MAP["autoupdate"]) > 0:
         # Autodownload
         if apt_pkg.config.find_i(softwareproperties.CONF_MAP["unattended"]) == 1\
-           and apt_pkg.config.find_i(softwareproperties.CONF_MAP["autodownload"]) == 1 :
+           and os.path.exists("/usr/bin/unattended-upgrade"):
             return softwareproperties.UPDATE_INST_SEC
         elif apt_pkg.config.find_i(softwareproperties.CONF_MAP["autodownload"]) == 1 and  \
              apt_pkg.config.find_i(softwareproperties.CONF_MAP["unattended"]) == 0:
@@ -178,9 +197,9 @@ class SoftwareProperties(object):
   def get_release_upgrades_policy(self):
     """
     return the release upgrade policy:
-     RELASE_UPGRADE_NEVER
-     RELASE_UPGRADE_NORMAL
-     RELASE_UPGRADE_LTS
+     RELEASE_UPGRADES_NORMAL,
+     RELEASE_UPGRADES_LTS,
+     RELEASE_UPGRADES_NEVER
     """
     # default (if no option is set) is NORMAL
     if not os.path.exists(self.RELEASE_UPGRADES_CONF):
@@ -189,7 +208,7 @@ class SoftwareProperties(object):
     parser.read(self.RELEASE_UPGRADES_CONF)
     if parser.has_option("DEFAULT","Prompt"):
       type = parser.get("DEFAULT","Prompt").lower()
-      for k, v in self.release_upgrades_policy_map.iteritems():
+      for k, v in self.release_upgrades_policy_map.items():
         if v == type:
           return k
     return self.RELEASE_UPGRADES_NORMAL
@@ -197,20 +216,21 @@ class SoftwareProperties(object):
   def set_release_upgrades_policy(self, i):
     """
     set the release upgrade policy:
-     RELASE_UPGRADE_NEVER
-     RELASE_UPGRADE_NORMAL
-     RELASE_UPGRADE_LTS
+     RELEASE_UPGRADES_NORMAL,
+     RELEASE_UPGRADES_LTS,
+     RELEASE_UPGRADES_NEVER
      """
     # we are note using ConfigParser.write() as it removes comments
     if not os.path.exists(self.RELEASE_UPGRADES_CONF):
       f = open(self.RELEASE_UPGRADES_CONF,"w")
-      f.write("[DEFAULT]\nprompt=%s\n"% self.release_upgrades_policy_map[i])
+      f.write("[DEFAULT]\nPrompt=%s\n"% self.release_upgrades_policy_map[i])
       return True
     f = open(self.RELEASE_UPGRADES_CONF,"r")
-    out = NamedTemporaryFile()
-    for line in map(string.strip, f.readlines()):
+    out = NamedTemporaryFile(mode="w+")
+    for line in f:
+      line = line.strip()
       if line.lower().startswith("prompt"):
-        out.write("prompt=%s\n" % self.release_upgrades_policy_map[i])
+        out.write("Prompt=%s\n" % self.release_upgrades_policy_map[i])
       else:
         out.write(line+"\n")
     out.flush()
@@ -245,14 +265,13 @@ class SoftwareProperties(object):
     if os.path.exists(self.popconfile):
         # read the current config and replace the corresponding settings
         # FIXME: should we check the other values, too?
-        lines = map(lambda line: re.sub(r'^(PARTICIPATE=)(".+?")',
-                                        '\\1"%s"' % value,
-                                        line),
-                    open(self.popconfile, "r").readlines())
+        with open(self.popconfile, "r") as popconfile:
+            lines = [re.sub(r'^(PARTICIPATE=)(".+?")', '\\1"%s"' % value, line)
+                     for line in popconfile]
     else:
         # create a new popcon config file
         m = md5()
-        m.update(open("/dev/urandom", "r").read(1024))
+        m.update(open("/dev/urandom", "rb").read(1024))
         id = m.hexdigest()
         lines = []
         lines.append("MY_HOSTID=\"%s\"\n" % id)
@@ -280,7 +299,7 @@ class SoftwareProperties(object):
         sources.extend(self.distro.main_sources)
         sources.extend(self.distro.child_sources)
         for source in sources:
-            if templates.has_key(source.template):
+            if source.template in templates:
                 for comp in source.comps:
                     templates[source.template].add(comp)
             else:
@@ -290,8 +309,8 @@ class SoftwareProperties(object):
         if len(self.distro.cdrom_sources) > 0:
             templates[self.distro.source_template] = self.distro.cdrom_comps
         for source in self.distro.source_code_sources:
-            if not templates.has_key(source.template) or \
-               (templates.has_key(source.template) and not \
+            if source.template not in templates or \
+               (source.template in templates and not \
                 (len(set(templates[source.template]) ^ set(source.comps)) == 0\
                  or (len(set(source.comps) ^ self.distro.enabled_comps) == 0))):
                 self.distro.get_source_code = False
@@ -305,19 +324,26 @@ class SoftwareProperties(object):
                            ("Comps:", source.comps),
                            ("Enabled:", not source.disabled),
                            ("Valid:", not source.invalid)]:
-        print " %s %s" % (label, value)
+        print(" %s %s" % (label, value))
     if source.template:
         for (label, value) in [("MatchURI:", source.template.match_uri),
                                ("BaseURI:", source.template.base_uri)]:
-            print " %s %s" % (label, value)
-    print "\n"
+            print(" %s %s" % (label, value))
+    print("\n")
 
   def massive_debug_output(self):
     """Print the complete sources.list""" 
-    print "START SOURCES.LIST:"
+    print("START SOURCES.LIST:")
     for source in self.sourceslist:
-        print source.str()
-    print "END SOURCES.LIST\n"
+        print(source.str())
+    print("END SOURCES.LIST\n")
+
+  def change_main_download_server(self, server):
+    """ change the main download server """
+    self.distro.default_server = server
+    res = self.distro.change_server(server)
+    self.set_modified_sourceslist()
+    return res
 
   def enable_component(self, comp):
     """Enable a component of the distro"""
@@ -329,8 +355,16 @@ class SoftwareProperties(object):
     self.distro.disable_component(comp) 
     self.set_modified_sourceslist()
 
+  def _find_template_from_string(self, name):
+    for template in self.distro.source_template.children:
+      if template.name == name:
+        return template
+
   def disable_child_source(self, template):
     """Enable a child repo of the distribution main repository"""
+    if isinstance(template, str):
+      template = self._find_template_from_string(template)
+
     for source in self.distro.child_sources:
         if source.template == template:
             self.sourceslist.remove(source)
@@ -341,6 +375,9 @@ class SoftwareProperties(object):
 
   def enable_child_source(self, template):
     """Enable a child repo of the distribution main repository"""
+    if isinstance(template, str):
+      template = self._find_template_from_string(template)
+
     # Use the currently selected mirror only if the child source
     # did not override the server
     if template.base_uri == None:
@@ -395,12 +432,34 @@ class SoftwareProperties(object):
         source_bkp = SourceEntry(line=source.line,file=source.file)
         self.sourceslist_backup.append(source_bkp)
   
+  def _find_source_from_string(self, line):
+    # ensure that we have a current list, it might have been changed underneath
+    # us
+    self.reload_sourceslist()
+
+    for source in self.sourceslist.list:
+      if str(source) == line:
+        return source
+    return None
+
   def toggle_source_use(self, source):
     """Enable or disable the selected channel"""
     #FIXME cdroms need to disable the comps in the childs and sources
+    if isinstance(source, str):
+      source = self._find_source_from_string(source)
     source.disabled = not source.disabled
     self.set_modified_sourceslist()
 
+  def replace_source_entry(self, old_entry, new_entry):
+    # find and replace, then write
+    for (index, entry) in enumerate(self.sourceslist.list):
+      if str(entry) == old_entry:
+        file = self.sourceslist.list[index].file
+        self.sourceslist.list[index] = SourceEntry(new_entry, file)
+        self.set_modified_sourceslist()
+        return True
+    return False
+
   def revert(self):
     """Revert all settings to the state when software-properties 
        was launched"""
@@ -433,19 +492,21 @@ class SoftwareProperties(object):
     """Render a nice output to show the source in a treeview"""
     if source.template == None:
         if source.comment:
-            contents = "<b>%s</b> (%s %s" % (escape(source.comment),
-                                              source.uri, source.dist)
+            contents = "<b>%s</b> %s" % (escape(source.comment).strip(),
+                                         source.dist)
             # Only show the components if there are more than one
             if len(source.comps) > 1:
                 for c in source.comps:
                     contents += " %s" % c
-            contents += ")"
+            if source.type in ("deb-src", "rpm-src"):
+                contents += " %s" % _("(Source Code)")
+            contents += "\n%s" % source.uri
         else:
             contents = "<b>%s %s</b>" % (source.uri, source.dist)
             for c in source.comps:
                 contents += " %s" % c
-        if source.type in ("deb-src", "rpm-src"):
-            contents += " %s" % _("(Source Code)")
+            if source.type in ("deb-src", "rpm-src"):
+                contents += " %s" % _("(Source Code)")
         return contents
     else:
         # try to make use of a corresponding template
@@ -472,7 +533,7 @@ class SoftwareProperties(object):
       has_template = 1
       has_comment = 1
       is_source = 1
-      revert_numbers = string.maketrans("0123456789", "9876543210")
+      revert_numbers = maketrans("0123456789", "9876543210")
       if source.template:
         has_template = 0
         desc = source.template.description
@@ -540,16 +601,16 @@ class SoftwareProperties(object):
   def write_config(self):
     """Write the current apt configuration to file"""
     # update the adept file as well if it is there
-    conffiles = ["/etc/apt/apt.conf.d/10periodic",
-                 "/etc/apt/apt.conf.d/20auto-upgrades",
-                 "/etc/apt/apt.conf.d/15adept-periodic-update"]
+    conffiles = [self.rootdir+"/etc/apt/apt.conf.d/10periodic",
+                 self.rootdir+"/etc/apt/apt.conf.d/20auto-upgrades",
+                 self.rootdir+"/etc/apt/apt.conf.d/15adept-periodic-update"]
 
     # check (beforehand) if one exists, if not create one
     for f in conffiles:
       if os.path.isfile(f):
         break
     else:
-      print "No config found, creating one"
+      print("No config found, creating one")
       open(conffiles[0], "w")
 
     # ensure /etc/cron.daily/apt is executable
@@ -557,8 +618,8 @@ class SoftwareProperties(object):
     if os.path.exists(ac):
       perm = os.stat(ac)[stat.ST_MODE]
       if not (perm & stat.S_IXUSR):
-        print "file '%s' not executable, fixing" % ac
-        os.chmod(ac, 0755)
+        print("file '%s' not executable, fixing" % ac)
+        os.chmod(ac, 0o755)
 
     # now update them
     for periodic in conffiles:
@@ -593,7 +654,7 @@ class SoftwareProperties(object):
     helper that checks if a given line is in the source list
     return the channel name or None if not found
     """
-    srcentry = SourceEntry(srcline)    
+    srcentry = SourceEntry(srcline)
     if os.path.exists(self.CHANNEL_PATH):
       for f in glob.glob("%s/*.list" % self.CHANNEL_PATH):
         for line in open(f):
@@ -604,20 +665,31 @@ class SoftwareProperties(object):
     return None
 
   def check_and_add_key_for_whitelisted_channels(self, srcline):
+    # This is maintained for any legacy callers
+    return self.check_and_add_key_for_whitelisted_shortcut(shortcut_handler(srcline))
+
+  def check_and_add_key_for_whitelisted_shortcut(self, shortcut):
     """
     helper that adds the gpg key of the channel to the apt
     keyring *if* the channel is in the whitelist
     /usr/share/app-install/channels or it is a public Launchpad PPA.
     """
+    (srcline, _fname) = shortcut.expand(
+        codename=self.distro.codename, distro=self.distro.id.lower())
     channel = self._is_line_in_whitelisted_channel(srcline)
     if channel:
       keyp = "%s/%s.key" % (self.CHANNEL_PATH, channel)
       self.add_key(keyp)
-    # FIXME: abstract this all alway into the ppa.py file
-    parsed_uri = urlparse(SourceEntry(srcline).uri)
-    if parsed_uri.netloc == 'ppa.launchpad.net':
-      worker = AddPPASigningKeyThread(parsed_uri.path)
-      worker.start()
+
+    cdata = (shortcut.add_key, {'keyserver': (self.options and
+                                              self.options.keyserver)})
+    def addkey_func():
+        func, kwargs = cdata
+        func(**kwargs)
+
+    worker = threading.Thread(target=addkey_func)
+    worker.start()
+    return worker
 
   def update_interface(self):
     " abstract interface to keep the UI alive "
@@ -634,74 +706,154 @@ class SoftwareProperties(object):
     repo = line.split()[0]
     try:
         areas = line.split(" ",1)[1]
-    except IndexError, e:
+    except IndexError:
         areas = "main"
     line = "deb %s %s %s" % ( repo, self.distro.codename, areas )
     return line
 
-  def add_source_from_line(self, line):
+  def add_source_from_line(self, line, enable_source_code=False):
     """
-    Add a source with the given apt line and auto-add
-    signing key if we have it in the whitelist
+    Add a source for the given line.
     """
+    return self.add_source_from_shortcut(
+        shortcut=shortcut_handler(line.strip()),
+        enable_source_code=enable_source_code)
+
+  def add_source_from_shortcut(self, shortcut, enable_source_code=False):
+    """
+    Add a source with the given shortcut and add the signing key if the
+    site is in whitelist or the shortcut implementer adds it.
+    """
+
     codenames = {}
+    codenames["flidas"] = "xenial" 
+    codenames["belenos"] = "trusty" 
+    codenames["toutatis"] = "precise" 
+    codenames["brigantia"] = "oneiric" 
     codenames["dagda"] = "natty"
     codenames["slaine"] = "maverick"
     codenames["taranis"] = "lucid"
     codenames["awen"] = "karmic" 
-    (deb_line, file) = expand_ppa_line(line.strip(), codenames[self.distro.codename])
+    (deb_line, file) = shortcut.expand(codename=codenames[self.distro.codename])
     deb_line = self.expand_http_line(deb_line)
-    debsrc_line = 'deb-src' + deb_line[3:]
+    debsrc_entry_type = 'deb-src' if enable_source_code else '# deb-src'
+    debsrc_line = debsrc_entry_type + deb_line[3:]
     new_deb_entry = SourceEntry(deb_line, file)
     new_debsrc_entry = SourceEntry(debsrc_line, file)
     if new_deb_entry.invalid or new_debsrc_entry.invalid:
       return False
-    self.check_and_add_key_for_whitelisted_channels(deb_line)
-    self.sourceslist.list.append(new_deb_entry)
-    self.sourceslist.list.append(new_debsrc_entry)
+    worker = self.check_and_add_key_for_whitelisted_shortcut(shortcut)
+    self.sourceslist.add(new_deb_entry.type,
+                         new_deb_entry.uri,
+                         new_deb_entry.dist,
+                         new_deb_entry.comps,
+                         comment=new_deb_entry.comment,
+                         file=new_deb_entry.file,
+                         architectures=new_deb_entry.architectures)
+    self.sourceslist.add(debsrc_entry_type,
+                         new_debsrc_entry.uri,
+                         new_debsrc_entry.dist,
+                         new_debsrc_entry.comps,
+                         comment=new_debsrc_entry.comment,
+                         file=new_debsrc_entry.file,
+                         architectures=new_debsrc_entry.architectures)
     self.set_modified_sourceslist()
+    if worker:
+        # wait for GPG key to be downloaded
+        worker.join(30)
+    if self.options and self.options.update:
+        import apt
+        cache = apt.Cache()
+        cache.update(sources_list=new_debsrc_entry.file)
     return True
 
-  def remove_source(self, source):
+  def remove_source(self, source, remove_source_code=True):
     """Remove the given source"""
+    if remove_source_code:
+      if isinstance(source, str):
+        # recall this method giving a SourceEntry as an argument
+        source = self._find_source_from_string(source)
+        self.remove_source(source, True)
+      elif source is not None:
+        self.remove_source(source, False)
+        # remove the deb-src lines (enabled or not) associated to
+        # this source entry
+        source = copy.copy(source)
+        source.type = 'deb-src'
+        source.disabled = True
+        self.remove_source(source, False)
+        source.disabled = False
+        self.remove_source(source, False)
+      return
+
+    # first find the source object if we got a string
+    if isinstance(source, str):
+      source = self._find_source_from_string(source)
+    if source is None:
+      return
     # if its a sources.list.d file and it contains only a single line
     # (the line that we just remove), then all references to that
     # file are gone and the file is not saved. we work around that
     # here
     if source.file != apt_pkg.config.find_file("Dir::Etc::sourcelist"):
       self.sourceslist.list.append(SourceEntry("", file=source.file))
-    self.sourceslist.remove(source)
+    try:
+      self.sourceslist.remove(source)
+    except ValueError:
+      # this exception is raised if trying to remove an entry that does
+      # not exist. in this case we suppress the error because there's no
+      # need to propagate it (the aim of this method is to ensure that
+      # the given entries are not listed in the sourceslist)
+      pass
     self.set_modified_sourceslist()
 
   def add_key(self, path):
     """Add a gnupg key to the list of trusted software vendors"""
-    if not isinstance(path, unicode):
-      path = unicode(path) # allows non-ascii filenames
     if not os.path.exists(path):
         return False
     try:
-        return self.apt_key.add(path)
+        res = self.apt_key.add(path)
+        self.KeysModified()
+        return res
     except:
         return False
 
   def add_key_from_data(self, keydata):
-    "Add a gnupg key from a data string (e.g. copy-n-paste)"
+    "Add a gnupg key from a utf-8 data string (e.g. copy-n-paste)"
     tmp = tempfile.NamedTemporaryFile()
-    tmp.write(keydata)
+    tmp.write(keydata.encode("utf-8"))
     tmp.flush()
     return self.add_key(tmp.name)
 
-  def remove_key(self, fingerprint):
+  def remove_key(self, keyid):
     """Remove a gnupg key from the list of trusted software vendors"""
     try:
-        self.apt_key.rm(fingerprint)
+        self.apt_key.rm(keyid)
+        self.KeysModified()
         return True
     except:
         return False
 
+  def update_keys(self):
+    """ Run apt-key update """
+    try:
+      self.apt_key.update()
+      self.KeysModified()
+      return True
+    except:
+      return False
+
+
+def shortcut_handler(shortcut):
+    for factory in _SHORTCUT_FACTORIES:
+        ret = factory(shortcut)
+        if ret is not None:
+            return ret
+
+    raise shortcuts.ShortcutException("Unable to handle input '%s'" % shortcut)
 
 
 if __name__ == "__main__":
   sp = SoftwareProperties()
-  print sp.get_release_upgrades_policy()
+  print(sp.get_release_upgrades_policy())
   sp.set_release_upgrades_policy(0)
