@@ -22,37 +22,32 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 #  USA
 
-import apt
+# FIXME: why does this pass kapps around?????
+
+from __future__ import absolute_import, print_function
+
 import apt_pkg
 import tempfile
 from gettext import gettext as _
 import os
-import re
-from xml.sax.saxutils import escape
-
-import sys
-import traceback
 import subprocess
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from PyQt4 import uic
-
-from PyKDE4.kdecore import ki18n, i18n, KUrl
-from PyKDE4.kdeui import KApplication, KMessageBox, KIcon, KWindowSystem
-from PyKDE4.kio import KFileDialog
+from PyQt5 import uic
+from PyQt5.QtCore import *
+from PyQt5.QtGui import *
+from PyQt5.QtWidgets import *
 
 import softwareproperties
 import softwareproperties.distro
 from softwareproperties.SoftwareProperties import SoftwareProperties
 import softwareproperties.SoftwareProperties
-from I18nHelper import *
+from .I18nHelper import *
 
 from aptsources.sourceslist import SourceEntry
-from DialogEdit import DialogEdit
-from DialogAdd import DialogAdd
-from DialogMirror import DialogMirror
-from CdromProgress import CdromProgress
+from .DialogEdit import DialogEdit
+from .DialogAdd import DialogAdd
+from .DialogMirror import DialogMirror
+from .CdromProgress import CdromProgress
 
 class SoftwarePropertiesKDEUI(QWidget):
   def __init__(self, datadir):
@@ -70,24 +65,26 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     self.datadir = datadir
 
     global kapp
-    kapp = KApplication()
+    kapp = QApplication.instance()
+    kapp.setWindowIcon(QIcon.fromTheme("applications-other"))
 
     self.userinterface = SoftwarePropertiesKDEUI(datadir)
-    self.userinterface.setWindowIcon(KIcon("applications-other"))
-    self.userinterface.button_auth_restore.setIcon(KIcon("edit-undo"))
-    self.userinterface.button_add_auth.setIcon(KIcon("list-add"))
-    self.userinterface.button_auth_remove.setIcon(KIcon("list-remove"))
-    self.userinterface.button_remove.setIcon(KIcon("list-remove"))
-    self.userinterface.button_edit.setIcon(KIcon("document-edit"))
-    self.userinterface.button_add.setIcon(KIcon("list-add"))
-    self.userinterface.button_add_cdrom.setIcon(KIcon("media-optical"))
+    self.userinterface.setWindowIcon(QIcon.fromTheme("applications-other"))
+    self.userinterface.button_auth_restore.setIcon(QIcon.fromTheme("edit-undo"))
+    self.userinterface.button_add_auth.setIcon(QIcon.fromTheme("list-add"))
+    self.userinterface.button_auth_remove.setIcon(QIcon.fromTheme("list-remove"))
+    self.userinterface.button_remove.setIcon(QIcon.fromTheme("list-remove"))
+    self.userinterface.button_edit.setIcon(QIcon.fromTheme("document-edit"))
+    self.userinterface.button_add.setIcon(QIcon.fromTheme("list-add"))
+    self.userinterface.button_add_cdrom.setIcon(QIcon.fromTheme("media-optical"))
     translate_widget(self.userinterface)
     self.userinterface.show()
-    if attachWinID is not None:
-        KWindowSystem.setMainWindow(self.userinterface, int(attachWinID))
+    # FIXME: winid not handled
+    #if attachWinID is not None:
+        #KWindowSystem.setMainWindow(self.userinterface, int(attachWinID))
 
     # rejected() signal from Close button
-    kapp.connect(self.userinterface.buttonBox, SIGNAL("rejected()"), self.on_close_button)
+    self.userinterface.buttonBox.rejected.connect(self.on_close_button)
 
     self.userinterface.buttonBox.button(QDialogButtonBox.Reset).setEnabled(False)
 
@@ -105,26 +102,26 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     self.show_isv_sources()
     self.show_cdrom_sources()
 
-    kapp.connect(self.userinterface.checkbutton_source_code, SIGNAL("clicked()"), self.on_checkbutton_source_code_toggled)
-    kapp.connect(self.userinterface.button_auth_restore, SIGNAL("clicked()"), self.on_restore_clicked)
-    kapp.connect(self.userinterface.button_add_auth, SIGNAL("clicked()"), self.add_key_clicked)
-    kapp.connect(self.userinterface.button_auth_remove, SIGNAL("clicked()"), self.remove_key_clicked)
-    kapp.connect(self.userinterface.checkbutton_popcon, SIGNAL("toggled(bool)"), self.on_checkbutton_popcon_toggled)
-    kapp.connect(self.userinterface.checkbutton_auto_update, SIGNAL("toggled(bool)"), self.on_auto_update_toggled)
-    kapp.connect(self.userinterface.combobox_update_interval, SIGNAL("currentIndexChanged(int)"), self.on_combobox_update_interval_changed)
-    kapp.connect(self.userinterface.button_remove, SIGNAL("clicked()"), self.on_remove_clicked)
-    kapp.connect(self.userinterface.button_edit, SIGNAL("clicked()"), self.on_edit_clicked)
-    kapp.connect(self.userinterface.button_add_cdrom, SIGNAL("clicked()"), self.on_button_add_cdrom_clicked)
-    kapp.connect(self.userinterface.button_add, SIGNAL("clicked()"), self.on_add_clicked)
-    kapp.connect(self.userinterface.treeview_sources, SIGNAL("itemChanged(QTreeWidgetItem*, int)"), self.on_isv_source_toggled)
-    kapp.connect(self.userinterface.treeview_sources, SIGNAL("itemClicked(QTreeWidgetItem*, int)"), self.on_treeview_sources_cursor_changed)
-    kapp.connect(self.userinterface.treeview_cdroms, SIGNAL("itemChanged(QTreeWidgetItem*, int)"), self.on_cdrom_source_toggled)
-    kapp.connect(self.userinterface.treeview2, SIGNAL("itemClicked(QTreeWidgetItem*, int)"), self.on_treeview_keys_cursor_changed)
+    self.userinterface.checkbutton_source_code.clicked.connect(self.on_checkbutton_source_code_toggled)
+    self.userinterface.button_auth_restore.clicked.connect(self.on_restore_clicked)
+    self.userinterface.button_add_auth.clicked.connect(self.add_key_clicked)
+    self.userinterface.button_auth_remove.clicked.connect(self.remove_key_clicked)
+    self.userinterface.checkbutton_popcon.toggled.connect(self.on_checkbutton_popcon_toggled)
+    self.userinterface.checkbutton_auto_update.toggled.connect(self.on_auto_update_toggled)
+    self.userinterface.combobox_update_interval.currentIndexChanged.connect(self.on_combobox_update_interval_changed)
+    self.userinterface.button_remove.clicked.connect(self.on_remove_clicked)
+    self.userinterface.button_edit.clicked.connect(self.on_edit_clicked)
+    self.userinterface.button_add_cdrom.clicked.connect(self.on_button_add_cdrom_clicked)
+    self.userinterface.button_add.clicked.connect(self.on_add_clicked)
+    self.userinterface.treeview_sources.itemChanged.connect(self.on_isv_source_toggled)
+    self.userinterface.treeview_sources.itemClicked.connect(self.on_treeview_sources_cursor_changed)
+    self.userinterface.treeview_cdroms.itemChanged.connect(self.on_cdrom_source_toggled)
+    self.userinterface.treeview2.itemClicked.connect(self.on_treeview_keys_cursor_changed)
     button_close = self.userinterface.buttonBox.button(QDialogButtonBox.Close)
-    button_close.setIcon(KIcon("dialog-close"))
+    button_close.setIcon(QIcon.fromTheme("dialog-close"))
     button_revert = self.userinterface.buttonBox.button(QDialogButtonBox.Reset)
-    button_revert.setIcon(KIcon("edit-undo"))
-    kapp.connect(button_revert, SIGNAL("clicked()"), self.on_button_revert_clicked)
+    button_revert.setIcon(QIcon.fromTheme("edit-undo"))
+    button_revert.clicked.connect(self.on_button_revert_clicked)
 
     self.init_distro()
     self.show_distro()
@@ -136,7 +133,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     if is_helpful != None:
       text = softwareproperties.distro.get_popcon_description(self.distro)
       text = text.replace("\n", "<br />") #silly GTK mixes HTML and normal white space
-      self.userinterface.label_popcon_desc.setText(utf8(text))
+      self.userinterface.label_popcon_desc.setText(text)
       self.userinterface.checkbutton_popcon.setChecked(is_helpful)
     else:
       self.userinterface.tabWidget.removeTab(4)
@@ -149,8 +146,9 @@ class SoftwarePropertiesKDE(SoftwareProperties):
   def init_release_upgrades(self):
     " setup the widgets that allow configuring the release upgrades "
     i = self.get_release_upgrades_policy()
+
     self.userinterface.combobox_release_upgrades.setCurrentIndex(i)
-    kapp.connect(self.userinterface.combobox_release_upgrades, SIGNAL("currentIndexChanged(int)"), self.on_combobox_release_upgrades_changed)
+    self.userinterface.combobox_release_upgrades.currentIndexChanged.connect(self.on_combobox_release_upgrades_changed)
 
   def init_auto_update(self):
     """ Set up the widgets that allow to configure the update automation """
@@ -161,17 +159,17 @@ class SoftwarePropertiesKDE(SoftwareProperties):
                                        2 : 7,
                                        3 : 14 }
 
-    self.userinterface.combobox_update_interval.insertItem(99, utf8(_("Daily")))
-    self.userinterface.combobox_update_interval.insertItem(99, utf8(_("Every two days")))
-    self.userinterface.combobox_update_interval.insertItem(99, utf8(_("Weekly")))
-    self.userinterface.combobox_update_interval.insertItem(99, utf8(_("Every two weeks")))
+    self.userinterface.combobox_update_interval.insertItem(99, _("Daily"))
+    self.userinterface.combobox_update_interval.insertItem(99, _("Every two days"))
+    self.userinterface.combobox_update_interval.insertItem(99, _("Weekly"))
+    self.userinterface.combobox_update_interval.insertItem(99, _("Every two weeks"))
 
     update_days = self.get_update_interval()
 
     # If a custom period is defined add a corresponding entry
     if not update_days in self.combobox_interval_mapping.values():
         if update_days > 0:
-            self.userinterface.combobox_update_interval.insertItem(99, utf8(_("Every %s days")) % update_days)
+            self.userinterface.combobox_update_interval.insertItem(99, _("Every %s days") % update_days)
             self.combobox_interval_mapping[4] = update_days
 
     for key in self.combobox_interval_mapping:
@@ -192,9 +190,9 @@ class SoftwarePropertiesKDE(SoftwareProperties):
       self.userinterface.radiobutton_updates_download.setEnabled(False)
       self.userinterface.radiobutton_updates_notify.setEnabled(False)
 
-    kapp.connect(self.userinterface.radiobutton_updates_download, SIGNAL("toggled(bool)"), self.set_update_automation_level)
-    kapp.connect(self.userinterface.radiobutton_updates_inst_sec, SIGNAL("toggled(bool)"), self.set_update_automation_level)
-    kapp.connect(self.userinterface.radiobutton_updates_notify, SIGNAL("toggled(bool)"), self.set_update_automation_level)
+    self.userinterface.radiobutton_updates_download.toggled.connect(self.set_update_automation_level)
+    self.userinterface.radiobutton_updates_inst_sec.toggled.connect(self.set_update_automation_level)
+    self.userinterface.radiobutton_updates_notify.toggled.connect(self.set_update_automation_level)
 
   def show_auto_update_level(self):
     """Represent the level of update automation in the user interface"""
@@ -213,11 +211,11 @@ class SoftwarePropertiesKDE(SoftwareProperties):
 
   def init_distro(self):
     # TRANS: %s stands for the distribution name e.g. Debian or Trisquel
-    text = u_("%s updates") % self.distro.id
+    text = _("%s updates") % self.distro.id
     text = text.replace("Trisquel", "Triskel")
     self.userinterface.groupBox_updates.setTitle(text)
     # TRANS: %s stands for the distribution name e.g. Debian or Trisquel
-    text = u_("%s Software") % self.distro.id
+    text = _("%s Software") % self.distro.id
     text = text.replace("Trisquel", "Triskel")
     self.userinterface.tabWidget.setTabText(0, text)
 
@@ -226,9 +224,9 @@ class SoftwarePropertiesKDE(SoftwareProperties):
         if isinstance(child, QGridLayout):
             self.vbox_dist_comps = child
         elif isinstance(child, QObject):
-            print "passing"
+            print("passing")
         else:
-            print "chidl: " + str(type(child))
+            print("child: " + str(type(child)))
             child.hide()
             del child
     self.checkboxComps = {}
@@ -237,12 +235,12 @@ class SoftwarePropertiesKDE(SoftwareProperties):
         #              first %s is the description of the component
         #              second %s is the code name of the comp, eg main, universe
         label = _("%s (%s)") % (comp.get_description(), comp.name)
-        checkbox = QCheckBox(utf8(label), self.userinterface.vbox_dist_comps_frame)
+        checkbox = QCheckBox(label, self.userinterface.vbox_dist_comps_frame)
         checkbox.setObjectName(comp.name)
         self.checkboxComps[checkbox] = comp
 
         # setup the callback and show the checkbutton
-        kapp.connect(checkbox, SIGNAL("clicked()"), self.on_component_toggled)
+        checkbox.clicked.connect(self.on_component_toggled)
         self.vbox_dist_comps.addWidget(checkbox)
 
     # Setup the checkbuttons for the child repos / updates
@@ -259,12 +257,12 @@ class SoftwarePropertiesKDE(SoftwareProperties):
         if template.type == "deb-src":
             continue
 
-        checkbox = QCheckBox(utf8(template.description), 
+        checkbox = QCheckBox(template.description, 
                              self.userinterface.vbox_updates_frame)
         checkbox.setObjectName(template.name)
         self.checkboxTemplates[checkbox] = template
         # setup the callback and show the checkbutton
-        kapp.connect(checkbox, SIGNAL("clicked()"), self.on_checkbutton_child_toggled)
+        checkbox.clicked.connect(self.on_checkbutton_child_toggled)
         self.vbox_updates.addWidget(checkbox)
 
     # If no components are enabled there will be no need for updates
@@ -274,7 +272,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
         self.userinterface.vbox_updates_frame.setEnabled(True)
 
     self.init_mirrors()
-    kapp.connect(self.userinterface.combobox_server, SIGNAL("activated(int)"), self.on_combobox_server_changed)
+    self.userinterface.combobox_server.activated.connect(self.on_combobox_server_changed)
 
   def init_mirrors(self):
     # Intiate the combobox which allows the user to specify a server for all
@@ -284,7 +282,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     self.userinterface.combobox_server.clear()
     for (name, uri, active) in self.distro.get_server_list():
         ##server_store.append([name, uri, False])
-        self.userinterface.combobox_server.addItem(utf8(name))
+        self.userinterface.combobox_server.addItem(name)
         self.mirror_urls.append(uri)
         if [name, uri] in self.seen_server:
             self.seen_server.remove([name, uri])
@@ -294,12 +292,12 @@ class SoftwarePropertiesKDE(SoftwareProperties):
             self.active_server = self.userinterface.combobox_server.count() - 1
             self.userinterface.combobox_server.setCurrentIndex(self.userinterface.combobox_server.count() - 1)
     for [name, uri] in self.seen_server:
-        self.userinterface.combobox_server.addItem(utf8(name))
+        self.userinterface.combobox_server.addItem(name)
         self.mirror_urls.append(uri)
     self.seen_server = seen_server_new
     # add a separator and the option to choose another mirror from the list
     ##FIXME server_store.append(["sep", None, True])
-    self.userinterface.combobox_server.addItem(utf8(_("Other...")))
+    self.userinterface.combobox_server.addItem(_("Other..."))
     self.other_mirrors_index = self.userinterface.combobox_server.count() - 1
 
   def show_distro(self):
@@ -360,24 +358,24 @@ class SoftwarePropertiesKDE(SoftwareProperties):
 
     # Output a lot of debug stuff
     if self.options.debug == True or self.options.massive_debug == True:
-        print "ENABLED COMPS: %s" % self.distro.enabled_comps
-        print "INTERNET COMPS: %s" % self.distro.download_comps
-        print "MAIN SOURCES"
+        print("ENABLED COMPS: %s" % self.distro.enabled_comps)
+        print("INTERNET COMPS: %s" % self.distro.download_comps)
+        print("MAIN SOURCES")
         for source in self.distro.main_sources:
             self.print_source_entry(source)
-        print "CHILD SOURCES"
+        print("CHILD SOURCES")
         for source in self.distro.child_sources:
             self.print_source_entry(source)
-        print "CDROM SOURCES"
+        print("CDROM SOURCES")
         for source in self.distro.cdrom_sources:
             self.print_source_entry(source)
-        print "SOURCE CODE SOURCES"
+        print("SOURCE CODE SOURCES")
         for source in self.distro.source_code_sources:
             self.print_source_entry(source)
-        print "DISABLED SOURCES"
+        print("DISABLED SOURCES")
         for source in self.distro.disabled_sources:
             self.print_source_entry(source)
-        print "ISV"
+        print("ISV")
         for source in self.sourceslist_visible:
             self.print_source_entry(source)
 
@@ -395,6 +393,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
   def on_combobox_release_upgrades_changed(self, combobox):
     """ set the release upgrades policy """
     i = self.userinterface.combobox_release_upgrades.currentIndex()
+
     self.set_release_upgrades_policy(i)
 
   def on_combobox_server_changed(self, combobox):
@@ -429,6 +428,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     Sync the components of all main sources (excluding cdroms),
     child sources and source code sources
     """
+    # FIXME: I find it rather questionable whether sender will work with pyqt doing weird signal handling
     tickBox = kapp.sender()
     if tickBox.checkState() == Qt.Checked:
         self.enable_component(str(tickBox.objectName()))
@@ -445,14 +445,11 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     """
     tickBox = kapp.sender()
     name = str(tickBox.objectName())
-    for template in self.distro.source_template.children:
-        if name == template.name:
-            aptTemplate = template
 
     if tickBox.checkState() == Qt.Checked:
-        self.enable_child_source(aptTemplate)
+        self.enable_child_source(name)
     elif tickBox.checkState() == Qt.Unchecked:
-        self.disable_child_source(aptTemplate)
+        self.disable_child_source(name)
     # no way to set back to mixed state so turn off tristate
     tickBox.setTristate(False)
 
@@ -557,7 +554,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     self.isv_sources = self.get_isv_sources()[:]
     self.show_sources(self.isv_sources, self.userinterface.treeview_sources)
 
-    if self.isv_sources < 1:
+    if not self.isv_sources or len(self.isv_sources) < 1:
         self.userinterface.button_remove.setEnabled(False)
         self.userinterface.button_edit.setEnabled(False)
 
@@ -577,7 +574,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
         contents = self.render_source(source)
         contents = contents.replace("<b>", "")
         contents = contents.replace("</b>", "")
-        item = QTreeWidgetItem([utf8(contents)])
+        item = QTreeWidgetItem([contents])
         if not source.disabled:
             item.setCheckState(0, Qt.Checked)
         else:
@@ -588,7 +585,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
   def show_keys(self):
     self.userinterface.treeview2.clear()
     for key in self.apt_key.list():
-      self.userinterface.treeview2.addTopLevelItem(QTreeWidgetItem([utf8(key)]))
+      self.userinterface.treeview2.addTopLevelItem(QTreeWidgetItem([key]))
 
     if self.userinterface.treeview_sources.topLevelItemCount() < 1:
       self.userinterface.button_auth_remove.setEnabled(False)
@@ -598,8 +595,8 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     if index != -1:
         value = self.combobox_interval_mapping[index]
         # Only write the key if it has changed
-        if not value == apt_pkg.Config.FindI(softwareproperties.CONF_MAP["autoupdate"]):
-            apt_pkg.Config.Set(softwareproperties.CONF_MAP["autoupdate"], str(value))
+        if not value == apt_pkg.config.find_i(softwareproperties.CONF_MAP["autoupdate"]):
+            apt_pkg.config.set(softwareproperties.CONF_MAP["autoupdate"], str(value))
             self.write_config()
 
   def on_auto_update_toggled(self):
@@ -631,7 +628,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
                        self.datadir, self.distro)
     line = dialog.run()
     if line != None:
-      self.add_source_from_line(utf8(line))
+      self.add_source_from_line(line)
       self.set_modified_sourceslist()
 
   def on_edit_clicked(self):
@@ -668,15 +665,14 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     if "SUDO_USER" in os.environ:
         home = os.path.expanduser("~%s" % os.environ["SUDO_USER"])
     url = KUrl.fromPath(home)
-    filename = KFileDialog.getOpenFileName(url, 'application/pgp-keys', self.userinterface, utf8(_("Import key")))
+    filename = KFileDialog.getOpenFileName(url, 'application/pgp-keys', self.userinterface, _("Import key"))
     if filename:
       if not self.add_key(filename):
         title = _("Error importing selected file")
         text = _("The selected file may not be a GPG key file " \
                 "or it might be corrupt.")
-        KMessageBox.sorry(self.userinterface,
-              utf8(text),
-              utf8(title))
+        #KMessageBox.sorry(self.userinterface, text, title)
+        QMessageBox.warning(self.userinterface, title, text)
       self.show_keys()
 
   def remove_key_clicked(self):
@@ -689,7 +685,8 @@ class SoftwarePropertiesKDE(SoftwareProperties):
       title = _("Error removing the key")
       text = _("The key you selected could not be removed. "
                "Please report this as a bug.")
-      KMessageBox.sorry(self.userinterface, utf8(text), utf8(title))
+      #KMessageBox.sorry(self.userinterface, text, title)
+      QMessageBox.warning(self.userinterface, title, text)
     self.show_keys()
 
   def on_restore_clicked(self):
@@ -703,16 +700,12 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     if self.modified_sourceslist == True and self.options.no_update == False:
         messageBox = QMessageBox(self.userinterface)
         messageBox.setIcon(QMessageBox.Information)
-        reloadButton = messageBox.addButton(utf8(_("Reload")), QMessageBox.AcceptRole)
+        reloadButton = messageBox.addButton(_("Reload"), QMessageBox.AcceptRole)
         messageBox.addButton(QMessageBox.Close)
-        text = _("<b><big>The information about available software is out-of-date</big></b>\n"
-               "\n"
-               "To install software and updates from newly added or changed sources, you "
-               "have to reload the information about available software.\n"
-               "\n"
-               "You need a working internet connection to continue.")
-        text = text.replace("\n", "<br />")
-        messageBox.setText(utf8(text))
+        primary = _("Your local copy of the software catalog is out of date.")
+        secondary = _("A new copy will be downloaded.")
+        text = "%s<br /><br /><small>%s</small>" % (primary, secondary)
+        messageBox.setText(text)
         messageBox.exec_()
         if (messageBox.clickedButton() == reloadButton):
                 cmd = ["/usr/bin/qapt-batch", "--update"]
@@ -723,23 +716,25 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     '''Show a dialog that allows to add a repository located on a CDROM
        or DVD'''
     # testing
-    #apt_pkg.Config.Set("APT::CDROM::Rename","true")
+    #apt_pkg.config.set("APT::CDROM::Rename","true")
 
-    saved_entry = apt_pkg.Config.Find("Dir::Etc::sourcelist")
+    saved_entry = apt_pkg.config.find("Dir::Etc::sourcelist")
     tmp = tempfile.NamedTemporaryFile()
-    apt_pkg.Config.Set("Dir::Etc::sourcelist",tmp.name)
+    apt_pkg.config.set("Dir::Etc::sourcelist",tmp.name)
     progress = CdromProgress(self.datadir, self, kapp)
-    cdrom = apt_pkg.GetCdrom()
+    cdrom = apt_pkg.Cdrom()
     # if nothing was found just return
     try:
-      res = cdrom.Add(progress)
-    except SystemError, msg:
+      res = cdrom.add(progress)
+    except SystemError as msg:
       title = _("CD Error")
-      text = _("<big><b>Error scanning the CD</b></big>\n\n%s")%msg
-      KMessageBox.sorry(self.userinterface, utf8(text), utf8(title))
+      primary = _("Error scanning the CD")
+      text = "%s\n\n<small>%s</small>" % (primary, msg)
+      #KMessageBox.sorry(self.userinterface, text, title)
+      QMessageBox.warning(self.userinterface, title, text)
       return
     finally:
-      apt_pkg.Config.Set("Dir::Etc::sourcelist",saved_entry)
+      apt_pkg.config.set("Dir::Etc::sourcelist",saved_entry)
       progress.close()
 
     if res == False:
@@ -749,7 +744,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     for x in open(tmp.name):
       line = x
     if line != "":
-      full_path = "%s%s" % (apt_pkg.Config.FindDir("Dir::Etc"),saved_entry)
+      full_path = "%s%s" % (apt_pkg.config.find_dir("Dir::Etc"),saved_entry)
       # insert cdrom source first, so that it has precedence over network sources
       self.sourceslist.list.insert(0, SourceEntry(line,full_path))
       self.set_modified_sourceslist()
